use serde_yaml;
use std::net::TcpListener;
use std::path::PathBuf;
use threadpool::ThreadPool;

mod static_http_server;

use static_http_server::ServerConfig;
use static_http_server::StaticHttpServer;

fn main() {
    let args: Vec<String> = std::env::args().collect();
    match args.len() {
        2 => {
            serve_with_config_path(&PathBuf::from(args[1].clone()));
        }
        _ => {
            println!("Usage:");
            println!("  {} <config.yaml>", args[0]);
        }
    }
}

fn serve_with_config_path(config: &PathBuf) {
    let server_config: ServerConfig =
        serde_yaml::from_str(std::fs::read_to_string(config).unwrap().as_str()).unwrap();
    serve_with_config(&server_config);
}

fn serve_with_config(config: &ServerConfig) {
    let bind_interface = format!("{}:{}", config.bind, config.port);
    let listener = TcpListener::bind(&bind_interface).unwrap();
    let pool = ThreadPool::new(config.threads);
    println!(
        "Listening on http://{} with {} threads",
        bind_interface, config.threads,
    );
    let mut static_server = StaticHttpServer::new(true, config.clone());
    for stream_result in listener.incoming() {
        static_server.check_for_changes();
        let stream = stream_result.unwrap();
        let cloned = static_server.clone();
        pool.execute(move || cloned.handle_stream(stream));
    }
}
